let svg = d3.select('svg.center');

let w = 120,
    ty = 62;

function draw(xt, h, shape, axes, notes) {
    let axis = axes[0];
    let rank = shape.length,
        cent = (shape.length-1)/2;

    let s = shape,
        shapes = [s],
        paths = d3.range(rank+1).map(i=>[i-s.length/2]),
        p = paths.slice(),
        reductions = [];
    axes.forEach(a => {
        s = s.slice();
        s.splice(a,1);
        shapes.push(s);
        let c = s.length/2;
        p.forEach((q,i)=>q.push(i-(i>a)-c));
        p.splice(a+1,1);
        reductions.push(a-c);
    });

    let part = d3.range(rank+1);

    let l = 80;
    function segment(xd, sign) {
        let yd = sign*(h-l),
            c  = yd/3,
            o  = (h-l)/10;
        return 'c' + [0,c+o,xd,yd-c+o,xd,yd].join(' ');
    }
    function path(x, up) {
        x = x.map(xi=>xi*w);
        if (up) { x.reverse(); }
        return d3.merge([
            [up ? 'H'+x[0] : 'M'+x[0]+' 0'],
            d3.range(x.length-1).map(i=>segment(x[i+1]-x[i], up?-1:1)),
            ['']
        ]).join('v'+(up?-l:l));
    }
    function region(sides) {
        let d = sides[1].length;
        return sides.map((p,j)=>path(
                p.length<d?p.concat([sides[1][d-1]]):p.slice(0,d),j
            ))
            .join('')+'Z';
    }

    let grp = svg.append('g')
        .attrs({'text-anchor':'middle',
                'font-family':'monospace',
                'font-size':60,
                'transform':'translate('+xt+',10)'});

    let cs = rank===3 ? [0,1,2] : [0,0.8,1.2,2];
    let backColor = (_,i) =>
        (c=>c.copy({l:1-(1-c.l)/2.8}))
            (d3.hsl(d3.interpolatePlasma(0.5+0.2*cs[i])));

    grp.selectAll().data(d3.range(rank).map(i => [paths[i],paths[i+1]])).enter()
        .append('path')
        .attr('fill', backColor)
        .attr('stoke', 'none')
        .attr('d', region);

    let sh = grp.selectAll().data(shapes).enter()
        .append('g').attr('transform',(_,i)=>'translate('+(w/2)*i+','+h*i+')');
    sh.append('rect').attrs({
            x:-(cent+1/2)*w, y:0, width:s=>s.length*w, height:l,
            fill:'rgba(240,240,240,0.25)'
        });
    sh.selectAll().data(s=>s).enter()
        .append('text').text(a=>a)
        .attr('x', (_,i)=>(i-cent)*w)
        .attr('y', ty);

    grp.selectAll().data(reductions).enter()
        .append('text').html('<tspan class="Type">+</tspan><tspan class="PreProc">⌿</tspan>')
        .attr('font-size',50)
        .attr('x', i=>i*w)
        .attr('y', (_,i)=>ty+(i+0.34)*h);

    grp.selectAll().data(paths.reverse()).enter()
        .append('path')
        .attrs(p=>{
            let s = p.length<axes.length+1;
            return {fill:'none', 'stroke-width':s?3.5:4,
                    stroke:s?'#667':'#334',
                    'stroke-dasharray':s?'8,10':null}
        })
        .attr('d', p=>path(p,false));

    if (notes) {
        let e = grp.selectAll().data(notes).enter()
            .append('g').attr('transform',(_,i)=>'translate(0,'+(i+1/2)*h+')');
        let xe = 510;
        let l = e.append('text').text(n=>n)
            .attr('text-anchor','end')
            .attr('font-family','sans-serif')
            .attr('font-size',40)
            .attr('x', xe)
            .attr('y', (_,i)=>ty-20)
            .nodes().map(n=>xe-n.getBoundingClientRect().width-5);
        e.append('line').datum((_,i)=>l[i])
            .attrs((x,i)=>({x1:x,x2:(i-0.5)*w/2, y1:30, y2:30,
                            stroke:'black','stroke-width':1}));
        grp.append('text')
            .attrs({x:0, y:ty-7, 'font-size':44})
            .text('×');
    }
}

draw( 400, 400, [1,  800,5], [1]  );
draw(1100, 200, [1,50,16,5], [1,1], ['Long rows','Small total size']);

svg.append('path')
    .attr('d', 'M644 210h100v-70l80 130l-80 130v-70h-100')
    .attrs({fill:'#c9e4f1', 'stroke-width':5, stroke:'#858693'});
